"use strict";
const typescript_ast_util_1 = require("typescript-ast-util");
const PLUGIN_NAME = 'typescript-plugin-print-ast';
const PRINT_AST_REFACTOR_ACTION_NAME = `${PLUGIN_NAME}-print-ast-refactor-action`;
const PRINT_PARENT_NODES_REFACTOR_ACTION_NAME = `${PLUGIN_NAME}-print-inheritance-refactor-action`;
let ts;
let info;
function init(modules) {
    ts = modules.typescript;
    function create(anInfo) {
        info = anInfo;
        info.project.projectService.logger.info(`${PLUGIN_NAME} created`);
        const proxy = Object.create(null);
        for (let k of Object.keys(info.languageService)) {
            const x = info.languageService[k];
            proxy[k] = (...args) => x.apply(info.languageService, args);
        }
        proxy.getApplicableRefactors = getApplicableRefactors;
        proxy.getEditsForRefactor = getEditsForRefactor;
        return proxy;
    }
    return { create };
}
let nodeAtCursor;
function getApplicableRefactors(fileName, positionOrRange) {
    const refactors = info.languageService.getApplicableRefactors(fileName, positionOrRange) || [];
    const sourceFile = info.languageService.getProgram().getSourceFile(fileName);
    if (!sourceFile) {
        return refactors;
    }
    nodeAtCursor = typescript_ast_util_1.findChildContainingRange(sourceFile, typescript_ast_util_1.positionOrRangeToRange(positionOrRange));
    if (!nodeAtCursor) {
        return refactors;
    }
    refactors.push({
        name: `${PLUGIN_NAME}-refactor-info`,
        description: 'Extract interface',
        actions: [
            { name: PRINT_AST_REFACTOR_ACTION_NAME, description: 'Print AST of selected ' + typescript_ast_util_1.getKindName(nodeAtCursor.kind) },
            { name: PRINT_PARENT_NODES_REFACTOR_ACTION_NAME, description: 'Print parent nodes of selected ' + typescript_ast_util_1.getKindName(nodeAtCursor.kind) }
        ],
    });
    return refactors;
}
function getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName) {
    const refactors = info.languageService.getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName);
    if ((actionName != PRINT_AST_REFACTOR_ACTION_NAME && actionName != PRINT_PARENT_NODES_REFACTOR_ACTION_NAME) || !nodeAtCursor) {
        return refactors;
    }
    let newText = '';
    if (actionName === PRINT_AST_REFACTOR_ACTION_NAME) {
        newText = `\`
AST of selected ${typescript_ast_util_1.getKindName(nodeAtCursor.kind)}:  
${typescript_ast_util_1.dumpAst(nodeAtCursor)}
\``;
    }
    else {
        let node = nodeAtCursor;
        const arr = [];
        newText += `\nParent AST Nodes of selected ${typescript_ast_util_1.getKindName(nodeAtCursor.kind)}:\n`;
        do {
            if (!node) {
                break;
            }
            arr.push(node);
            node = node.parent;
        } while (node);
        let level = 1;
        newText += arr.reverse().map(n => {
            return typescript_ast_util_1.printNode(n, -1, level++);
        }).join('\n');
        newText = '\n`\n' + newText + '\n`\n';
    }
    return {
        edits: [{
                fileName,
                textChanges: [{
                        span: { start: nodeAtCursor.getSourceFile().getEnd(), length: newText.length },
                        newText: newText
                    }],
            }],
        renameFilename: undefined,
        renameLocation: undefined,
    };
}
module.exports = init;
